--------------------------------------------------------------------------------
--
--                COPYRIGHT (C) 2000-2001 - LSI LOGIC.
--    All rights reserved. Reproduction in whole or part is prohibited
--          without the written permission of the copyright owner.
--
--------------------------------------------------------------------------------

---------------------------------------------------------------------------
-- Entity declaration of 'PROC_BLOCK_MATCH'
---------------------------------------------------------------------------

library ieee          ;

use ieee.std_logic_1164.all    ;
use ieee.std_logic_arith.all   ;

use WORK.package_g12.all ;

ENTITY proc_block_match IS
  port(
    GCLEAR         ,
    CK             ,
    PT_SYNC        : IN  std_ulogic ;
    HP_IN_BUS      ,
    CL_IN_BUS      : IN  std_ulogic_vector(7 downto 0) ;
    HP_OUT_BUS     ,
    CL_OUT_BUS     ,
    ER7_IN_DELAYED : OUT std_ulogic_vector(7 downto 0) ;
    P0_ENERGY      ,
    P1_ENERGY      , 
    P2_ENERGY      , 
    P3_ENERGY      : OUT std_ulogic_vector(13 downto 0)  ) ;
END proc_block_match ;

---------------------------------------------------------------------------
-- Architecture 'beh' of 'PROC_BLOCK_MATCH'
---------------------------------------------------------------------------

architecture beh of proc_block_match is


    -- Component declaration --
    component proc_timer
    port(
         GCLEAR  ,
         CK      ,
         SYNC    : IN  std_ulogic ;
         PH_SW   ,
         PL_REF  ,
         PL_PRED : OUT std_ulogic  ) ;
    end component ;
    
    component proc_mv_sec
    port(
         GCLEAR    ,
         CK        ,
         PL_REF    ,
         PL_PRED   : IN  std_ulogic ;
         SHIFT_IN  ,
         ELAB_IN   : IN  std_ulogic_vector(7 downto 0) ;
         SHIFT_OUT ,
         ELAB_OUT  : OUT std_ulogic_vector(7 downto 0) ;
         ELAB_REG  ,
         PRED0_REG ,
         PRED1_REG ,
         PRED2_REG ,
         PRED3_REG : OUT array_8bit(7 downto 0)         ) ;
    end component ;
    
    component proc_mux_z8
    port(
         GCLEAR      ,
         CK          ,
         MUX_SEL     : IN  std_ulogic ;
         SHIFT_IN_D0 ,
         SHIFT_IN_D1 : IN  std_ulogic_vector(7 downto 0) ;
         SHIFT_OUT   : OUT std_ulogic_vector(7 downto 0)   ) ;
    end component ;
    
    component proc_comb_sec
    port(
         GCLEAR ,
         CK     : IN  std_ulogic ;
    
         ELAB_COL0 ,
         ELAB_COL1 ,
         ELAB_COL2 ,
         ELAB_COL3 ,
         ELAB_COL4 ,
         ELAB_COL5 ,
         ELAB_COL6 ,
         ELAB_COL7 ,
    
         PRED0_COL0 ,
         PRED1_COL0 ,
         PRED2_COL0 ,
         PRED3_COL0 ,
         PRED0_COL1 ,
         PRED1_COL1 ,
         PRED2_COL1 ,
         PRED3_COL1 ,
         PRED0_COL2 ,
         PRED1_COL2 ,
         PRED2_COL2 ,
         PRED3_COL2 ,
         PRED0_COL3 ,
         PRED1_COL3 ,
         PRED2_COL3 ,
         PRED3_COL3 ,
         PRED0_COL4 ,
         PRED1_COL4 ,
         PRED2_COL4 ,
         PRED3_COL4 ,
         PRED0_COL5 ,
         PRED1_COL5 ,
         PRED2_COL5 ,
         PRED3_COL5 ,
         PRED0_COL6 ,
         PRED1_COL6 ,
         PRED2_COL6 ,
         PRED3_COL6 ,
         PRED0_COL7 ,
         PRED1_COL7 ,
         PRED2_COL7 ,
         PRED3_COL7 : IN  array_8bit(7 downto 0) ;
         P0_ENERGY  ,
         P1_ENERGY  , 
         P2_ENERGY  , 
         P3_ENERGY  : OUT std_ulogic_vector(13 downto 0) ) ;
    end component ;
    
        
    -- Signal declaration --
    signal ph_sw   ,
           pl_ref  ,
           pl_pred : std_ulogic ;
    
    signal cl_bus_z8_delay ,
           hp_bus_z8_delay : array_8bit(7 downto 0) ;
    
    signal column0_elab    ,
           column0_pred0   ,
           column0_pred1   ,
           column0_pred2   ,
           column0_pred3   ,
           
           column1_elab    ,
           column1_pred0   ,
           column1_pred1   ,
           column1_pred2   ,
           column1_pred3   ,
           
           column2_elab    ,
           column2_pred0   ,
           column2_pred1   ,
           column2_pred2   ,
           column2_pred3   ,   
           
           column3_elab    ,
           column3_pred0   ,
           column3_pred1   ,
           column3_pred2   ,
           column3_pred3   ,
           
           column4_elab    ,
           column4_pred0   ,
           column4_pred1   ,
           column4_pred2   ,
           column4_pred3   ,
           
           column5_elab    ,
           column5_pred0   ,
           column5_pred1   ,
           column5_pred2   ,
           column5_pred3   ,
           
           column6_elab    ,
           column6_pred0   ,
           column6_pred1   ,
           column6_pred2   ,
           column6_pred3   ,
           
           column7_elab    ,
           column7_pred0   ,
           column7_pred1   ,
           column7_pred2   ,
           column7_pred3   : array_8bit(7 downto 0) ;
           
    signal column0_elab_reg_out  ,
           column0_shift_reg_out ,
           
           column1_shift_reg_in  ,
           column1_elab_reg_in   ,
           column1_elab_reg_out  ,
           column1_shift_reg_out ,
           
           column2_shift_reg_in  ,
           column2_elab_reg_in   ,
           column2_elab_reg_out  ,
           column2_shift_reg_out ,
           
           column3_shift_reg_in  ,
           column3_elab_reg_in   ,
           column3_elab_reg_out  ,
           column3_shift_reg_out ,
           
           column4_shift_reg_in  ,
           column4_elab_reg_in   ,
           column4_elab_reg_out  ,
           column4_shift_reg_out ,
           
           column5_shift_reg_in  ,
           column5_elab_reg_in   ,
           column5_elab_reg_out  ,
           column5_shift_reg_out ,
           
           column6_shift_reg_in  ,
           column6_elab_reg_in   ,
           column6_elab_reg_out  ,
           column6_shift_reg_out ,
           
           column7_shift_reg_in  ,
           column7_elab_reg_in   ,
           column7_elab_reg_out  ,
           column7_shift_reg_out : std_ulogic_vector(7 downto 0) ;

    signal hp_out_sampled : std_ulogic_vector(7 downto 0) ;            

BEGIN

    -- Processor Timer --
xa0 : proc_timer
      port map(
         GCLEAR  => GCLEAR  ,
         CK      => CK      ,
         SYNC    => PT_SYNC ,
         PH_SW   => ph_sw   ,
         PL_REF  => pl_ref  ,
         PL_PRED => pl_pred  ) ;
                  
               
    -- Sequential process --
p0 : process(GCLEAR, CK)
     begin
     
        if( GCLEAR = '0' ) then
        
           cl_bus_z8_delay <= (others => (others => '0')) ;
           hp_out_sampled  <= (others => '0') ;
           CL_OUT_BUS      <= (others => '0') ;
           ER7_IN_DELAYED  <= (others => '0') ;
           hp_bus_z8_delay <= (others => (others => '0')) ; 
           
        elsif( CK'event and CK = '1' ) then
        
           -- z8 delay on CL_IN_BUS
           cl_bus_z8_delay <= cl_bus_z8_delay(6 downto 0) & CL_IN_BUS ;
           
           -- out_bus
           if(ph_sw = '1') then
              hp_out_sampled <= column7_elab_reg_out  ;
              CL_OUT_BUS     <= column7_shift_reg_out ;
           else
              hp_out_sampled <= column7_shift_reg_out ;
              CL_OUT_BUS     <= column7_elab_reg_out  ;
           end if ;
           
           -- ER7_IN_DELAYED
           ER7_IN_DELAYED    <= column7_elab_reg_in   ;
           
           -- z8 delay on hp_out_sampled
           hp_bus_z8_delay <= hp_bus_z8_delay(6 downto 0) & hp_out_sampled ;
           
        end if ;
        
     end process ;
     
     -- ass.
     HP_OUT_BUS <= hp_bus_z8_delay(7) ;
     

        -- Movement structure --
    
    -- Column n. 0 --
xc0 : proc_mv_sec
      port map(
         GCLEAR    => GCLEAR                ,
         CK        => CK                    ,
         PL_REF    => pl_ref                ,
         PL_PRED   => pl_pred               ,
         SHIFT_IN  => HP_IN_BUS             ,
         ELAB_IN   => cl_bus_z8_delay(7)    ,
         SHIFT_OUT => column0_shift_reg_out ,
         ELAB_OUT  => column0_elab_reg_out  ,
         ELAB_REG  => column0_elab          , 
         PRED0_REG => column0_pred0         ,
         PRED1_REG => column0_pred1         ,
         PRED2_REG => column0_pred2         ,
         PRED3_REG => column0_pred3          ) ;

         
    -- Column n. 1 --
xz_shift_c1 : 
      proc_mux_z8
      port map(
         GCLEAR      => GCLEAR                ,
         CK          => CK                    ,
         MUX_SEL     => ph_sw                 ,
         SHIFT_IN_D0 => column0_shift_reg_out ,
         SHIFT_IN_D1 => column0_elab_reg_out  ,
         SHIFT_OUT   => column1_shift_reg_in   ) ;             

xz_elab_c1 : 
      proc_mux_z8
      port map(
         GCLEAR      => GCLEAR                ,
         CK          => CK                    ,
         MUX_SEL     => ph_sw                 ,
         SHIFT_IN_D0 => column0_elab_reg_out  ,
         SHIFT_IN_D1 => column0_shift_reg_out ,
         SHIFT_OUT   => column1_elab_reg_in    ) ;

xc1 : proc_mv_sec
      port map(
         GCLEAR    => GCLEAR                ,
         CK        => CK                    ,
         PL_REF    => pl_ref                ,
         PL_PRED   => pl_pred               ,
         SHIFT_IN  => column1_shift_reg_in  ,         
         ELAB_IN   => column1_elab_reg_in   ,
         SHIFT_OUT => column1_shift_reg_out ,
         ELAB_OUT  => column1_elab_reg_out  ,
         ELAB_REG  => column1_elab          , 
         PRED0_REG => column1_pred0         ,
         PRED1_REG => column1_pred1         ,
         PRED2_REG => column1_pred2         ,
         PRED3_REG => column1_pred3          ) ;


    -- Column n. 2 --
xz_shift_c2 : 
      proc_mux_z8
      port map(
         GCLEAR      => GCLEAR                ,
         CK          => CK                    ,
         MUX_SEL     => ph_sw                 ,
         SHIFT_IN_D0 => column1_shift_reg_out ,
         SHIFT_IN_D1 => column1_elab_reg_out  ,
         SHIFT_OUT   => column2_shift_reg_in   ) ;             

xz_elab_c2 : 
      proc_mux_z8
      port map(
         GCLEAR      => GCLEAR                ,
         CK          => CK                    ,
         MUX_SEL     => ph_sw                 ,
         SHIFT_IN_D0 => column1_elab_reg_out  ,
         SHIFT_IN_D1 => column1_shift_reg_out ,
         SHIFT_OUT   => column2_elab_reg_in    ) ;

xc2 : proc_mv_sec
      port map(
         GCLEAR    => GCLEAR                ,
         CK        => CK                    ,
         PL_REF    => pl_ref                ,
         PL_PRED   => pl_pred               ,
         SHIFT_IN  => column2_shift_reg_in  ,         
         ELAB_IN   => column2_elab_reg_in   ,
         SHIFT_OUT => column2_shift_reg_out ,
         ELAB_OUT  => column2_elab_reg_out  ,
         ELAB_REG  => column2_elab          , 
         PRED0_REG => column2_pred0         ,
         PRED1_REG => column2_pred1         ,
         PRED2_REG => column2_pred2         ,
         PRED3_REG => column2_pred3          ) ;



    -- Column n. 3 --
xz_shift_c3 : 
      proc_mux_z8
      port map(
         GCLEAR      => GCLEAR                ,
         CK          => CK                    ,
         MUX_SEL     => ph_sw                 ,
         SHIFT_IN_D0 => column2_shift_reg_out ,
         SHIFT_IN_D1 => column2_elab_reg_out  ,
         SHIFT_OUT   => column3_shift_reg_in   ) ;             

xz_elab_c3 : 
      proc_mux_z8
      port map(
         GCLEAR      => GCLEAR                ,
         CK          => CK                    ,
         MUX_SEL     => ph_sw                 ,
         SHIFT_IN_D0 => column2_elab_reg_out  ,
         SHIFT_IN_D1 => column2_shift_reg_out ,
         SHIFT_OUT   => column3_elab_reg_in    ) ;

xc3 : proc_mv_sec
      port map(
         GCLEAR    => GCLEAR                ,
         CK        => CK                    ,
         PL_REF    => pl_ref                ,
         PL_PRED   => pl_pred               ,
         SHIFT_IN  => column3_shift_reg_in  ,         
         ELAB_IN   => column3_elab_reg_in   ,
         SHIFT_OUT => column3_shift_reg_out ,
         ELAB_OUT  => column3_elab_reg_out  ,
         ELAB_REG  => column3_elab          , 
         PRED0_REG => column3_pred0         ,
         PRED1_REG => column3_pred1         ,
         PRED2_REG => column3_pred2         ,
         PRED3_REG => column3_pred3          ) ;



    -- Column n. 4 --
xz_shift_c4 : 
      proc_mux_z8
      port map(
         GCLEAR      => GCLEAR                ,
         CK          => CK                    ,
         MUX_SEL     => ph_sw                 ,
         SHIFT_IN_D0 => column3_shift_reg_out ,
         SHIFT_IN_D1 => column3_elab_reg_out  ,
         SHIFT_OUT   => column4_shift_reg_in   ) ;             

xz_elab_c4 : 
      proc_mux_z8
      port map(
         GCLEAR      => GCLEAR                ,
         CK          => CK                    ,
         MUX_SEL     => ph_sw                 ,
         SHIFT_IN_D0 => column3_elab_reg_out  ,
         SHIFT_IN_D1 => column3_shift_reg_out ,
         SHIFT_OUT   => column4_elab_reg_in    ) ;

xc4 : proc_mv_sec
      port map(
         GCLEAR    => GCLEAR                ,
         CK        => CK                    ,
         PL_REF    => pl_ref                ,
         PL_PRED   => pl_pred               ,
         SHIFT_IN  => column4_shift_reg_in  ,         
         ELAB_IN   => column4_elab_reg_in   ,
         SHIFT_OUT => column4_shift_reg_out ,
         ELAB_OUT  => column4_elab_reg_out  ,
         ELAB_REG  => column4_elab          , 
         PRED0_REG => column4_pred0         ,
         PRED1_REG => column4_pred1         ,
         PRED2_REG => column4_pred2         ,
         PRED3_REG => column4_pred3          ) ;



    -- Column n. 5 --
xz_shift_c5 : 
      proc_mux_z8
      port map(
         GCLEAR      => GCLEAR                ,
         CK          => CK                    ,
         MUX_SEL     => ph_sw                 ,
         SHIFT_IN_D0 => column4_shift_reg_out ,
         SHIFT_IN_D1 => column4_elab_reg_out  ,
         SHIFT_OUT   => column5_shift_reg_in   ) ;             

xz_elab_c5 : 
      proc_mux_z8
      port map(
         GCLEAR      => GCLEAR                ,
         CK          => CK                    ,
         MUX_SEL     => ph_sw                 ,
         SHIFT_IN_D0 => column4_elab_reg_out  ,
         SHIFT_IN_D1 => column4_shift_reg_out ,
         SHIFT_OUT   => column5_elab_reg_in    ) ;

xc5 : proc_mv_sec
      port map(
         GCLEAR    => GCLEAR                ,
         CK        => CK                    ,
         PL_REF    => pl_ref                ,
         PL_PRED   => pl_pred               ,
         SHIFT_IN  => column5_shift_reg_in  ,         
         ELAB_IN   => column5_elab_reg_in   ,
         SHIFT_OUT => column5_shift_reg_out ,
         ELAB_OUT  => column5_elab_reg_out  ,
         ELAB_REG  => column5_elab          , 
         PRED0_REG => column5_pred0         ,
         PRED1_REG => column5_pred1         ,
         PRED2_REG => column5_pred2         ,
         PRED3_REG => column5_pred3          ) ;



    -- Column n. 6 --
xz_shift_c6 : 
      proc_mux_z8
      port map(
         GCLEAR      => GCLEAR                ,
         CK          => CK                    ,
         MUX_SEL     => ph_sw                 ,
         SHIFT_IN_D0 => column5_shift_reg_out ,
         SHIFT_IN_D1 => column5_elab_reg_out  ,
         SHIFT_OUT   => column6_shift_reg_in   ) ;             

xz_elab_c6 : 
      proc_mux_z8
      port map(
         GCLEAR      => GCLEAR                ,
         CK          => CK                    ,
         MUX_SEL     => ph_sw                 ,
         SHIFT_IN_D0 => column5_elab_reg_out  ,
         SHIFT_IN_D1 => column5_shift_reg_out ,
         SHIFT_OUT   => column6_elab_reg_in    ) ;

xc6 : proc_mv_sec
      port map(
         GCLEAR    => GCLEAR                ,
         CK        => CK                    ,
         PL_REF    => pl_ref                ,
         PL_PRED   => pl_pred               ,
         SHIFT_IN  => column6_shift_reg_in  ,         
         ELAB_IN   => column6_elab_reg_in   ,
         SHIFT_OUT => column6_shift_reg_out ,
         ELAB_OUT  => column6_elab_reg_out  ,
         ELAB_REG  => column6_elab          , 
         PRED0_REG => column6_pred0         ,
         PRED1_REG => column6_pred1         ,
         PRED2_REG => column6_pred2         ,
         PRED3_REG => column6_pred3          ) ;



    -- Column n. 7 --
xz_shift_c7 : 
      proc_mux_z8
      port map(
         GCLEAR      => GCLEAR                ,
         CK          => CK                    ,
         MUX_SEL     => ph_sw                 ,
         SHIFT_IN_D0 => column6_shift_reg_out ,
         SHIFT_IN_D1 => column6_elab_reg_out  ,
         SHIFT_OUT   => column7_shift_reg_in   ) ;             

xz_elab_c7 : 
      proc_mux_z8
      port map(
         GCLEAR      => GCLEAR                ,
         CK          => CK                    ,
         MUX_SEL     => ph_sw                 ,
         SHIFT_IN_D0 => column6_elab_reg_out  ,
         SHIFT_IN_D1 => column6_shift_reg_out ,
         SHIFT_OUT   => column7_elab_reg_in    ) ;

xc7 : proc_mv_sec
      port map(
         GCLEAR    => GCLEAR                ,
         CK        => CK                    ,
         PL_REF    => pl_ref                ,
         PL_PRED   => pl_pred               ,
         SHIFT_IN  => column7_shift_reg_in  ,         
         ELAB_IN   => column7_elab_reg_in   ,
         SHIFT_OUT => column7_shift_reg_out ,
         ELAB_OUT  => column7_elab_reg_out  ,
         ELAB_REG  => column7_elab          , 
         PRED0_REG => column7_pred0         ,
         PRED1_REG => column7_pred1         ,
         PRED2_REG => column7_pred2         ,
         PRED3_REG => column7_pred3          ) ;


        -- Computing stages --
xa1 : proc_comb_sec
      port map(
         GCLEAR    => GCLEAR                ,
         CK        => CK                    ,
    
         ELAB_COL0 => column0_elab          ,
         ELAB_COL1 => column1_elab          ,
         ELAB_COL2 => column2_elab          ,
         ELAB_COL3 => column3_elab          ,
         ELAB_COL4 => column4_elab          ,
         ELAB_COL5 => column5_elab          ,
         ELAB_COL6 => column6_elab          ,
         ELAB_COL7 => column7_elab          ,
    
         PRED0_COL0 => column0_pred0        ,
         PRED1_COL0 => column0_pred1        ,
         PRED2_COL0 => column0_pred2        ,
         PRED3_COL0 => column0_pred3        ,
         PRED0_COL1 => column1_pred0        ,
         PRED1_COL1 => column1_pred1        ,
         PRED2_COL1 => column1_pred2        ,
         PRED3_COL1 => column1_pred3        ,
         PRED0_COL2 => column2_pred0        ,
         PRED1_COL2 => column2_pred1        ,
         PRED2_COL2 => column2_pred2        ,
         PRED3_COL2 => column2_pred3        ,
         PRED0_COL3 => column3_pred0        ,
         PRED1_COL3 => column3_pred1        ,
         PRED2_COL3 => column3_pred2        ,
         PRED3_COL3 => column3_pred3        ,
         PRED0_COL4 => column4_pred0        ,
         PRED1_COL4 => column4_pred1        ,
         PRED2_COL4 => column4_pred2        ,
         PRED3_COL4 => column4_pred3        ,
         PRED0_COL5 => column5_pred0        ,
         PRED1_COL5 => column5_pred1        ,
         PRED2_COL5 => column5_pred2        ,
         PRED3_COL5 => column5_pred3        ,
         PRED0_COL6 => column6_pred0        ,
         PRED1_COL6 => column6_pred1        ,
         PRED2_COL6 => column6_pred2        ,
         PRED3_COL6 => column6_pred3        ,
         PRED0_COL7 => column7_pred0        ,
         PRED1_COL7 => column7_pred1        ,
         PRED2_COL7 => column7_pred2        ,
         PRED3_COL7 => column7_pred3        ,
         P0_ENERGY  => P0_ENERGY            ,
         P1_ENERGY  => P1_ENERGY            , 
         P2_ENERGY  => P2_ENERGY            , 
         P3_ENERGY  => P3_ENERGY             ) ;
         
         
END beh ;

