--------------------------------------------------------------------------------
--
--                COPYRIGHT (C) 2000-2001 - LSI LOGIC.
--    All rights reserved. Reproduction in whole or part is prohibited
--          without the written permission of the copyright owner.
--
--------------------------------------------------------------------------------

---------------------------------------------------------------------------
-- Entity declaration of 'PROC_FCOMP_SEC'
---------------------------------------------------------------------------

library ieee                 ;

use ieee.std_logic_1164.all  ;
use ieee.std_logic_arith.all ;

use WORK.package_g12.all     ;

ENTITY proc_fcomp_sec IS
  port(
       CK         ,
       GCLEAR     : IN  std_ulogic ;
       SUM_COL_P0 ,
       SUM_COL_P1 ,
       SUM_COL_P2 ,
       SUM_COL_P3 : IN array_12bit(7 downto 0) ;
       P0_FSUM    ,
       P1_FSUM    ,
       P2_FSUM    ,
       P3_FSUM    : OUT std_ulogic_vector(13 downto 0) ) ;
END proc_fcomp_sec ;

---------------------------------------------------------------------------
-- Architecture 'beh' of 'PROC_FCOMP_SEC'
---------------------------------------------------------------------------

architecture beh of proc_fcomp_sec is

    -- Type declaration --
    type array_12bit_unsigned is array(0 to 3) of unsigned(11 downto 0) ;           
    type array_13bit_unsigned is array(0 to 1) of unsigned(12 downto 0) ;
        
    -- Signal declaration --
    signal sum_column_p0_stage1 ,
           sum_column_p1_stage1 ,
           sum_column_p2_stage1 ,
           sum_column_p3_stage1 : array_12bit_unsigned ;
           
    signal sum_column_p0_stage2 ,
           sum_column_p1_stage2 ,
           sum_column_p2_stage2 ,
           sum_column_p3_stage2 : array_13bit_unsigned ;       
    
    signal sum_p0_stagef ,
           sum_p1_stagef ,
           sum_p2_stagef ,
           sum_p3_stagef : unsigned(13 downto 0) ;

BEGIN


      -- First state of computing --
p0 : process( SUM_COL_P0, SUM_COL_P1, SUM_COL_P2, SUM_COL_P3)
     variable Temp_p0 ,
              Temp_p1 ,
              Temp_p2 ,
              Temp_p3 : unsigned(12 downto 0) ; 
     begin

            
       for k in 0 to 3 loop

         -- PRED0 --
         -------------------------------------------------------------------------
         Temp_p0 := ( '0' & unsigned(SUM_COL_P0(k*2)(10 downto 0)) & SUM_COL_P0(k*2+1)(11)  )
                                               +
                       ( unsigned( SUM_COL_P0(k*2+1)(10 downto 0) ) & '1' ) ;
                            
         sum_column_p0_stage1(k) <= Temp_p0(12 downto 1) ;                               
         -------------------------------------------------------------------------
         
         -- PRED1 --
         -------------------------------------------------------------------------
         Temp_p1 := ( '0' & unsigned(SUM_COL_P1(k*2)(10 downto 0)) & SUM_COL_P1(k*2+1)(11) )
                                               +
                       ( unsigned(SUM_COL_P1(k*2+1)(10 downto 0) ) & '1' ) ;
                            
                                                        
         sum_column_p1_stage1(k) <= Temp_p1(12 downto 1) ;
         
         -------------------------------------------------------------------------
                  
         -- PRED2 --
         -------------------------------------------------------------------------
         Temp_p2 := ( '0' & unsigned(SUM_COL_P2(k*2)(10 downto 0)) & SUM_COL_P2(k*2+1)(11) )
                                               +         
                       ( unsigned( SUM_COL_P2(k*2+1)(10 downto 0)) & '1' ) ;
                            
         sum_column_p2_stage1(k) <= Temp_p2(12 downto 1) ;
        -------------------------------------------------------------------------
         
         -- PRED3 --
         -------------------------------------------------------------------------
         Temp_p3 := ( '0' & unsigned(SUM_COL_P3(k*2)(10 downto 0)) & SUM_COL_P3(k*2+1)(11) )
                                               +         
                       ( unsigned( SUM_COL_P3(k*2+1)(10 downto 0)) & '1' ) ;
                                     
         sum_column_p3_stage1(k) <= Temp_p3(12 downto 1) ;
         -------------------------------------------------------------------------
         
       end loop ;
              
     end process ;


      -- Second state of computing --
p1 : process( sum_column_p0_stage1, sum_column_p1_stage1,
              sum_column_p2_stage1, sum_column_p3_stage1,
              SUM_COL_P0, SUM_COL_P1, SUM_COL_P2, SUM_COL_P3)
     variable Temp_p0 ,
              Temp_p1 ,
              Temp_p2 ,
              Temp_p3 : unsigned(13 downto 0) ;
     begin
        for k in 0 to 1 loop
        
         -- PRED0 --
  -------------------------------------------------------------------------
  Temp_p0 := ( '0' & sum_column_p0_stage1(k*2) & SUM_COL_P0(k*4)(11) )
                                         +
                      ( sum_column_p0_stage1(k*2+1) & '1' ) ;
                                                               
  sum_column_p0_stage2(k) <= Temp_p0(13 downto 1) ;
  -------------------------------------------------------------------------
         
         -- PRED1 --
  -------------------------------------------------------------------------
  Temp_p1 := ( '0' & sum_column_p1_stage1(k*2) & SUM_COL_P1(k*4)(11) )
                                         +  
                      ( sum_column_p1_stage1(k*2+1) & '1' ) ;
                      
  sum_column_p1_stage2(k) <= Temp_p1(13 downto 1) ;
  -------------------------------------------------------------------------
           
         -- PRED2 --
  -------------------------------------------------------------------------
  Temp_p2 := ( '0' & sum_column_p2_stage1(k*2) & SUM_COL_P2(k*4)(11) )
                                         +
                      ( sum_column_p2_stage1(k*2+1) & '1' ) ; 
                        
  sum_column_p2_stage2(k) <= Temp_p2(13 downto 1) ;
  -------------------------------------------------------------------------
           
         -- PRED3 --
  -------------------------------------------------------------------------
  Temp_p3 := ( '0' & sum_column_p3_stage1(k*2) & SUM_COL_P3(k*4)(11) )
                                         +
                      ( sum_column_p3_stage1(k*2+1) & '1' ) ;  
  
  sum_column_p3_stage2(k) <= Temp_p3(13 downto 1) ;
  -------------------------------------------------------------------------
                                    
       end loop ;
                
     end process ;
     
    -- Final stage --
p2 : process( sum_column_p0_stage2, sum_column_p1_stage2,
              sum_column_p2_stage2, sum_column_p3_stage2,
              SUM_COL_P0, SUM_COL_P1,
              SUM_COL_P2, SUM_COL_P3 )
     variable Tempf_p0 ,
              Tempf_p1 ,
              Tempf_p2 ,
              Tempf_p3 : unsigned(14 downto 0) ;
     begin
     ---- PRED0 ----
     Tempf_p0 := ( '0' & sum_column_p0_stage2(0) & SUM_COL_P0(2)(11) )
                                      +
                     ( sum_column_p0_stage2(1) & '1' ) ;
                     
     sum_p0_stagef <= Tempf_p0(14 downto 1) + SUM_COL_P0(6)(11) ;                   
     ---------------
     
     ---- PRED1 ----
     Tempf_p1 := ( '0' & sum_column_p1_stage2(0) & SUM_COL_P1(2)(11) )
                                      +
                     ( sum_column_p1_stage2(1) & '1' ) ;
                     
     sum_p1_stagef <= Tempf_p1(14 downto 1) + SUM_COL_P1(6)(11) ;                   
     ---------------
     
     ---- PRED2 ----
     Tempf_p2 := ( '0' & sum_column_p2_stage2(0) & SUM_COL_P2(2)(11) )
                                      +
                     ( sum_column_p2_stage2(1) & '1' ) ;
                     
     sum_p2_stagef <= Tempf_p2(14 downto 1) + SUM_COL_P2(6)(11) ;                   
     ---------------
     
     ---- PRED3 ----
     Tempf_p3 := ( '0' & sum_column_p3_stage2(0) & SUM_COL_P3(2)(11) )
                                      +
                     ( sum_column_p3_stage2(1) & '1' ) ;
                     
     sum_p3_stagef <= Tempf_p3(14 downto 1) + SUM_COL_P3(6)(11) ;                   
     ---------------                   
                                        
     end process ; 
     
     -- Pipeline output ass. --
p3 : process(CK, GCLEAR)
     begin
        if(GCLEAR = '0') then
        
           P0_FSUM <= (others => '0') ;
           P1_FSUM <= (others => '0') ;
           P2_FSUM <= (others => '0') ;
           P3_FSUM <= (others => '0') ;
          
        elsif(CK'event and CK = '1') then

           P0_FSUM <= std_ulogic_vector( sum_p0_stagef ) ;
           P1_FSUM <= std_ulogic_vector( sum_p1_stagef ) ;
           P2_FSUM <= std_ulogic_vector( sum_p2_stagef ) ;
           P3_FSUM <= std_ulogic_vector( sum_p3_stagef ) ;
     
        end if ;
        
     end process ;
     
            


     
     
END beh ;

