--------------------------------------------------------------------------------
--
--                COPYRIGHT (C) 2000-2001 - LSI LOGIC.
--    All rights reserved. Reproduction in whole or part is prohibited
--          without the written permission of the copyright owner.
--
--------------------------------------------------------------------------------

---------------------------------------------------------------------------
-- Entity declaration of 'PROC_TIMER'
---------------------------------------------------------------------------

library ieee          ;

use ieee.std_logic_1164.all    ;
use ieee.std_logic_arith.all   ;

use WORK.package_g12.all ;

ENTITY proc_timer IS
  port(
    GCLEAR  ,
    CK      ,
    SYNC    : IN  std_ulogic ;
    PH_SW   ,
    PL_REF  ,
    PL_PRED : OUT std_ulogic  ) ;
END proc_timer ;

---------------------------------------------------------------------------
-- Architecture 'beh' of 'PROC_TIMER'
---------------------------------------------------------------------------

architecture beh of proc_timer is

    
    -- Signal declaration --
    signal q1_st : unsigned(2 downto 0) ;
    signal q2_st : unsigned(3 downto 0) ;
    signal q1_tc ,
           q2_tc : std_ulogic ;


BEGIN

    -- Sequential process --
p0 : process(GCLEAR, CK)
     begin
     
        if( GCLEAR = '0' ) then
        
           q1_st   <= (others => '0') ;
           q2_st   <= (others => '0') ;
           PH_SW   <= '0' ;
           PL_REF  <= '0' ;
           PL_PRED <= '0' ;
           
        elsif( CK'event and CK = '1' ) then
        
          -- lsb counter stage
          if(SYNC = '1') then
             q1_st <= (others => '0') ;
          else
             q1_st <= q1_st + 1 ;
          end if ;
          
          -- msb counter stage
          if(SYNC = '1') then
             q2_st <= (others => '0') ;
          else
             if(q1_tc = '1') then
                q2_st <= q2_st + 1 ;
             end if ;
          end if ;
          
          -- PH_SW
          PH_SW   <= q2_st(0) ;
          
          -- PL_REF
          PL_REF  <= not( q2_st(0) ) and q1_tc ;
          
          -- PL_PRED
          PL_PRED <= q1_tc and q2_tc ;
          
        end if ;
        
     end process ;
     
     -- combinatorial
     q1_tc <= '1' when q1_st = unsigned'("111")  else '0' ;
     q2_tc <= '1' when q2_st = unsigned'("1111") else '0' ;


END beh ;

